userSource = "BPNET Hybrid"

logName = system.tag.read("[default]Configuration/Debug/LoggerName").value
log = system.util.getLogger(logName)
##########################################################################################
##########################################################################################
def getUserScheduleModifications(userName, startDate, endDate):

	#This function will return any exceptional dates that a user has had added or removed for vacation etc.

	hdr = ['username', 'name', 'Start Date', 'End Date', 'Available', 'Note']
	rows = []

	user = system.user.getUser(userSource, userName)
	name = user.get('firstname') + user.get('lastname')
	scheduleMod = user.getScheduleAdjustments()
	for item in scheduleMod:
		beginDate = item.getStart()
		finishDate = item.getEnd()
		if (beginDate > startDate and beginDate < endDate) or finishDate > startDate and finishDate < endDate:  
			isAvailable = item.isAvailable()
			note = item.getNote()
			
			rows.append([userName, name, beginDate, finishDate, isAvailable, note])
			
	return system.dataset.toDataSet(hdr, rows)

##########################################################################################
def getUserByRole(siteCode, userRole):

	#Pass in a role i.e. 'NC090_MES_Operator' to return an arraylist of users that have this role
	
	userList = []

	users = system.user.getUsers(userSource)
	for user in users:
		roles = user.getRoles()
		for role in roles:
			if siteCode in role and userRole in role:
				userList.append(user)
			
	return userList
###########################################################################################	
def getRolesBySiteCode(siteCode):

	#Pass in a sitePath and return the roles
	roleList = []

	if siteCode is not None:	
		roles = system.user.getRoles(userSource)
		for role in roles:
			if siteCode in role:
				roleList.append(role)
			
	return roleList
##########################################################################################
def getUserSchedule(userName):

	user = system.user.getUser(userSource, userName)
	schedule = user.get('schedule')

	return schedule
##########################################################################################
def setUserSchedule(userName, scheduleName):

	user = system.user.getUser(userSource, userName)
	user.set('schedule', scheduleName)
	system.user.editUser(userSource, user)
	system.tag.write("[default]Chemical/User Schedules/update", True)

	return	
###########################################################################################
def getSchedule(scheduleName, startDate, endDate):

#	print scheduleName, startDate, endDate

	from java.util import Calendar, Date
	cal = Calendar.getInstance()
	startDate = system.date.midnight(startDate)
	cal.setTime(startDate)	
	schedule = system.user.getSchedule(scheduleName)
	
	hdr = ['Schedule', 'Start Date', 'End Date']
	rows = []	
	
	if schedule is not None:
		schedules = []					
								
		for i in range(system.date.daysBetween(startDate,endDate) + 1):
			timeline = schedule.getScheduleForDay(cal)
			## timeline will return time as if gateway were in GMT timezone and the calendar day were Jan. 1, 1970
			## add the start and end values to the Long value of midnight for the current day (in Gateway's time)
			if timeline is not None:
				segs = timeline.getSegments()
				midnight = cal.getTimeInMillis()
				for seg in segs:
					start = Date(seg.getStart()+midnight)
					end = Date(seg.getEnd()+midnight)
					rows.append([schedule.getName(), start, end, ])
			
			cal.add(Calendar.DAY_OF_MONTH, 1)
		
	else:
		log.warn(scheduleName + " does not exist")

	data = system.dataset.toDataSet(hdr, rows)
	data = system.dataset.sort(data, "Start Date", True)
	
	return data
##########################################################################################	
def getScheduledUsers(siteName, startDate):

	#This function will return a dataset of users who are available based on the startDate passed
	#It takes into account any schedule modifications 
	#The passed in siteName e.g. 'Conway' is used to filter users who are part of the 'Conway Shift A/B/C or D/ composite schedules

	hdr = ['username', 'name', 'schedule']
	rows = []
	users = system.user.getScheduledUsers(userSource, startDate)
	for user in users:
		scheduleName = user.get(user.Schedule)
		if siteName + ' Shift' in scheduleName:
			username = user.get(user.Username)
			name = user.get(user.LastName).replace(",","") + " " + user.get(user.FirstName).replace(",","")
			rows.append([username, name, scheduleName])
			
	data = system.dataset.toDataSet(hdr, rows)	
	
	return data	
##############################################################################################
def copyShiftSchedule(fromSchedule, toSchedule):

#	#JUST EXAMPLE CODE HERE
#	fromSchedule = "Conway Shift A4"
#	toSchedule = "Conway Shift B4"
	
	returnMsg = "" 
		
	mySchedule = system.user.getSchedule(fromSchedule)
	
	if mySchedule is None:
		returnMsg = "Couldn't find a schedule with this name"
	elif mySchedule.getType() != "basic schedule":
		returnMsg = "Can only copy basic schedules"
	else:
#		mySchedule.setObserveHolidays(False)
		mySchedule.setName(toSchedule)
		response = system.user.addSchedule(mySchedule)
		warnings = response.getWarns()
		errors = response.getErrors()
		infos = response.getInfos()
		if len(warnings) > 0:
			returnMsg = returnMsg + str(warnings[0])
			
		if len(errors) > 0:
			returnMsg = returnMsg + str(errors[0])

		if len(infos) > 0:
			returnMsg = returnMsg + str(infos[0])
			
			
#		print infos, errors, warnings	
		
	return returnMsg
##############################################################################################
def getSiteSchedules(siteName):

	#This function will get a list of shifts that a user can be set to based on the Site they are working at
	#We are enforcing some stuff here. We expect the schedule to be named 'Conway Shift A' or 'Eugene Something Shift A'.
	#The logic will ignore any entries that have '_comp' in the name and any shift name that has more than one char in 
	#the final string i.e. 'Conway Shift A1'

	schedules = []

	scheduleList = system.user.getScheduleNames()
	for scheduleName in scheduleList:
		if scheduleName.find(siteName) != -1:	
			if scheduleName.upper().find('_COMP') == -1:
				if len(scheduleName.split(" ")[len(scheduleName.split(" "))-1]) == 1:
					schedules.append(scheduleName)
			
	return schedules
	
##############################################################################################	
def getAvailableUsers(sitePath, startDate, endDate, userRole):

	#returns only users who are scheduled for a certain time period
#	userRole = "Operator"

	log.info('getAvailableUsers() called for ' + str(sitePath) + " " + str(startDate) + " " + str(endDate) + " " + str(userRole)) 

	###### Create a filter dataset of users that work at this site ##########
	userData = system.tag.read("[default]Chemical/User Schedules/userData").value
	deleteRows = []
	for row in range(userData.rowCount):
		if userData.getValueAt(row, "eqPath") != sitePath:
			deleteRows.append(row)
	
	userData = system.dataset.deleteRows(userData, deleteRows)
	
	####### Create a dataset of schedules for this site and time period #############
	siteName = shared.Kanoa.Utilities.getItemFromEqPath(sitePath, 'SITE')
	scheduleNameList = shared.Kanoa.Users.getSiteSchedules(siteName)
	
	scheduleData = None
	for scheduleName in scheduleNameList:
		data = shared.Kanoa.Users.getSchedule(scheduleName, startDate, endDate)
		shared.Kanoa.Utilities.printDataset(data)
		if scheduleData is None:
			if data.rowCount > 0:
				scheduleData = data
		else:
			scheduleData = system.dataset.appendDataset(scheduleData, data)

	##### Now build the Calender events dataset ##########################
	#The header looks a bit odd because I am appending it to the MO Data in the Calendar View
	hdr = ['startDate', 'endDate', 'displayColor', 'display', 'uuid', 'opDefName', 'eventType', 'eqPath', 'Fixed', 'MO#', 'Product']
	rows = []
	displayColorList = ['#D9D900', '#D97700', '#00AC00', '#00ACAC']	
	userList = []

	for row in range(scheduleData.rowCount):
		schedule = scheduleData.getValueAt(row, 'Schedule')	
		
		userString = "<HTML>" + schedule + "<BR>"
		for rowx in range(userData.rowCount):
			if userData.getValueAt(rowx, "Schedule") == schedule:
				userList.append(userData.getValueAt(rowx, 'Username'))
				userString = userString + userData.getValueAt(rowx, 'User') + '<BR>' 
		
		userString = userString + "</HTML>"
		
		try:	#I've only specified 4 different shift, so if more shifts are selected, we'll go for the exception color 
			displayColor = displayColorList[scheduleNameList.index(schedule)]
		except:
			displayColor = 	'#D900D9'

		display = userString			
		beginDate = system.date.addSeconds(scheduleData.getValueAt(row, 'Start Date'), 1)			
		finishDate = system.date.addSeconds(scheduleData.getValueAt(row, 'End Date'),-1)				
		rows.append([beginDate, finishDate, displayColor, display, '', '', 'userSchedule', '', '', '', ''])		
		
	userList = list(set(userList))
	
	for user in userList:
		userScheduleData = shared.Kanoa.Users.getUserScheduleModifications(user, startDate, endDate)	
		for row in range(userScheduleData.rowCount):
			beginDate = system.date.addSeconds(userScheduleData.getValueAt(row, 'Start Date'), 1)
			finishDate = system.date.addSeconds(userScheduleData.getValueAt(row, 'End Date'),-1)
			note = userScheduleData.getValueAt(row, 'Note')
			isAvailable = userScheduleData.getValueAt(row, 'Available')
			name = userScheduleData.getValueAt(row, 'name')
			display = name + " " + str(note)
		
			if isAvailable:
				displayColor = '#00FF00'
			else:
				displayColor = "#FF0000"	
			
			rows.append([beginDate, finishDate, displayColor, display, '', '', 'userScheduleAdjust', '', '', '', ''])	

	data = system.dataset.toDataSet(hdr, rows)	
	
	return data
	
#def getAvailableUsers(sitePath, startDate, endDate, userRole):
#
#	#returns only users who are scheduled for a certain time period
##	userRole = "Operator"
#
#	log.info('getAvailableUsers() called for ' + str(sitePath) + " " + str(startDate) + " " + str(endDate) + " " + str(userRole)) 
#
#	###### Create a filter dataset of users that work at this site ##########
#	userData = system.tag.read("[default]Chemical/User Schedules/userData").value
#	deleteRows = []
#	for row in range(userData.rowCount):
#		if userData.getValueAt(row, "eqPath") != sitePath:
#			deleteRows.append(row)
#	
#	userData = system.dataset.deleteRows(userData, deleteRows)
#	
#	####### Create a dataset of schedules for this site and time period #############
#	siteName = shared.Kanoa.Utilities.getItemFromEqPath(sitePath, 'SITE')
#	scheduleNameList = shared.Kanoa.Users.getSiteSchedules(siteName)
#	
#	scheduleData = None
#	for scheduleName in scheduleNameList:
#		data = shared.Kanoa.Users.getSchedule(scheduleName, startDate, endDate)
#		if scheduleData is None:
#			scheduleData = data
#		elif scheduleData.rowCount == 0:
#			scheduleData = data
#		else:
#			scheduleData = system.dataset.appendDataset(scheduleData, data)
#
#	##### Now build the Calender events dataset ##########################
#	#The header looks a bit odd because I am appending it to the MO Data in the Calendar View
#	hdr = ['startDate', 'endDate', 'displayColor', 'display', 'uuid', 'opDefName', 'eventType', 'eqPath', 'Fixed', 'MO#', 'Product']
#	rows = []
#	displayColorList = ['#D9D900', '#D97700', '#00AC00', '#00ACAC']	
#	userList = []
#
#	for row in range(scheduleData.rowCount):
#		schedule = scheduleData.getValueAt(row, 'Schedule')	
#		
#		userString = "<HTML>" + schedule + "<BR>"
#		for rowx in range(userData.rowCount):
#			if userData.getValueAt(rowx, "Schedule") == schedule:
#				userList.append(userData.getValueAt(rowx, 'Username'))
#				userString = userString + userData.getValueAt(rowx, 'User') + '<BR>' 
#		
#		userString = userString + "</HTML>"
#		
#		try:	#I've only specified 4 different shift, so if more shifts are selected, we'll go for the exception color 
#			displayColor = displayColorList[scheduleNameList.index(schedule)]
#		except:
#			displayColor = 	'#D900D9'
#
#		display = userString			
#		beginDate = system.date.addSeconds(scheduleData.getValueAt(row, 'Start Date'), 1)			
#		finishDate = system.date.addSeconds(scheduleData.getValueAt(row, 'End Date'),-1)				
#		rows.append([beginDate, finishDate, displayColor, display, '', '', 'userSchedule', '', '', '', ''])		
#		
#	userList = list(set(userList))
#	
#	for user in userList:
#		userScheduleData = shared.Kanoa.Users.getUserScheduleModifications(user, startDate, endDate)	
#		for row in range(userScheduleData.rowCount):
#			beginDate = system.date.addSeconds(userScheduleData.getValueAt(row, 'Start Date'), 1)
#			finishDate = system.date.addSeconds(userScheduleData.getValueAt(row, 'End Date'),-1)
#			note = userScheduleData.getValueAt(row, 'Note')
#			isAvailable = userScheduleData.getValueAt(row, 'Available')
#			name = userScheduleData.getValueAt(row, 'name')
#			display = name + " " + str(note)
#		
#			if isAvailable:
#				displayColor = '#00FF00'
#			else:
#				displayColor = "#FF0000"	
#			
#			rows.append([beginDate, finishDate, displayColor, display, '', '', 'userScheduleAdjust', '', '', '', ''])	
#
#	data = system.dataset.toDataSet(hdr, rows)	
#	
#	return data
	
##############################################################################################
#		USEFUL INFO REGARDING USERS
##############################################################################################

#userName = system.security.getUsername()
#user = system.user.getUser("", userName)
#print user.get('firstname') + " " + user.get('lastname')
#print user.getContactInfo()
#print user.getRoles()
#schedule = user.get('schedule')

#def printResponse(responseList):
#    if len(responseList) > 0:
#        for response in responseList:
#            print "", response
#    else:
#        print " None" 
#
## Make a brand new 'blank' user. Not saved until we, well, save
#username = event.source.parent.getComponent('Text Field').text
#user = system.user.getNewUser("", "myAwesomeUser")
#
## Let's fill in some fields. Note we have two ways to access property names
#user.set("firstname", "Naomi")
#user.set(user.LastName, "Nagata")
#user.set("password", "1234567890")
#
## We can add contact info one at a time. Up to the script user to make sure the type is legit
#user.addContactInfo("email", "naomi@roci.com")
#
##we can add a lot of contact info
#contactInfo = {"email":"ignition_user@mycompany.com","sms": "5551212"}
#user.addContactInfo(contactInfo)
#
## We can delete contact info. Only deletes if both fields match.
#user.removeContactInfo("sms", "5551212")
#
## we can add a role. If the role doesn't already exist, user save will fail, depending on user source
#user.addRole("mechanic")
#
## we can add a lot of roles
#roles = ["Administrator", "prisoner"] 
#user.addRoles(roles)
#
## and we can remove a role
#user.removeRole("prisoner")
#
## we can add a schedule adjustment too
#date2 = system.date.now()
#date1 = system.date.midnight(date2)
#user.addScheduleAdjustment(date1, date2, False, "An adjustment note")
#
## we can make a bunch of adjustments and add them en-masse
#date3 = system.date.addDays(date2, -4)
#adj1 = system.user.createScheduleAdjustment(date3, date2, True, "Another note")
#adj2 = system.user.createScheduleAdjustment(date3, date1, False, "")
#user.addScheduleAdjustments([adj1, adj2])
#
## and we can remove a schedule adjustment. All fields must match.
#user.removeScheduleAdjustment(date1, date2, True, "Some other note")
#
## finally need to save our new user and print responses
#response = system.user.addUser("", user)
#
#warnings = response.getWarns()
#print "Warnings are:"
#printResponse(warnings)
# 
#errors = response.getErrors()
#print "Errors are:"
#printResponse(errors)
# 
#infos = response.getInfos()
#print "Infos are:"
#printResponse(infos)

##############################################################################################
#		OLD CODE STUBS
##########################################################################################	
##########################################################################################	
#def getShiftSchedules(eqPath, startDate, endDate):
#	headers = ['startDate', 'endDate', 'displayColor', 'display', 'uuid', 'opDefName', 'eventType']	
#	data = []	
#	
#	if eqPath != '':
#	
#		from java.util import Calendar, Date, TimeZone
#		from java.lang import Long, Integer
#		
#		shiftColors = ['ff8aff', 'ac00ac', '800080', 'ffccff']	#We are presuming that there are only four shifts ever	
#		
#		obj = system.mes.loadMESObjectByEquipmentPath(eqPath)
#		scheduleList = obj.getIgnitionSchedule() #Get the Ignition Shift Schedules set for this Line
#		
#		if len(scheduleList) > 0:
#			scheduleList = scheduleList.split(',')
#			for i in range(len(scheduleList)):
#				s = system.user.getSchedule(scheduleList[i])	
#				name = s.getName()
#				colorString = shiftColors[i]
#					
#				models = []
#				
#				if s.getType().lower() == "composite schedule":
#					for model in s.getModels():
#						models.append(model)
#				else:
#					models.append(s)
#				for model in models:
#					cal = Calendar.getInstance()
#					weekSize = 7
#					cal.setTime(startDate)			
#					i=0				## loop until next month
#					while i < system.date.daysBetween(startDate,endDate) + 1:
#						i+=1
#						try:
#							timeline = model.getScheduleForDay(cal)
#							## timeline will return time as if gateway were in GMT timezone and the calendar day were Jan. 1, 1970
#							## add the start and end values to the Long value of midnight for the current day (in Gateway's time)
#							segs = timeline.getSegments()
#							midnight = cal.getTimeInMillis()
#							for seg in segs:
#								start = Date(seg.getStart()+midnight)
#								end = Date(seg.getEnd()+midnight)
#								data.append([start,end,colorString,name,'','', 'SHIFT EVENT'])
#						except:
#							pass
#						cal.add(Calendar.DAY_OF_MONTH, 1)
#						
#	return system.dataset.toDataSet(headers,data)